unit Magvarip;

interface

Function MParts(E1: Real): Real;
Function MLati(E1: Real): Real;

implementation

Const
  PiDiv = 0.01745329251994322958; {Pi / 180}
  LogN10 = 2.30258509299404568; {Log Nat.(10)}
  MPartConst1 = 7915.704468; {Meridional P. 1}
  MPartConst2 = 23.0133633;  {Meridional P. 2}
  Zero = 0.000000000000001;  {Virtual zero}

Function XSin(E1: Real): Real;  
begin
  XSin := Sin(E1 * PiDiv);
end;

Function XCos(E1: Real): Real;  
begin
  XCos := Cos(E1 * PiDiv);
end;

Function XTan(E1: Real): Real;  
begin
{  If E1 = 90 then E1 := 89.999;
  if E1 = 270 then E1 := 269.999;}
  If (E1 <> 90) and (E1 <> 270) then
    XTan := XSin(E1) / XCos(E1)
  else
    If E1 = 90 then
      Xtan := 999999999999.0
    else { = 270}
      XTan := -999999999999.0;
end;

Function XLog(E1: Real): Real;  
begin
  if E1 < 0 then E1 := E1 * -1;
  if E1 = 0 then XLog := -50 else
    XLog := Ln(E1) / LogN10;
end;

Function XATan(E1: Real): Real;  
begin
  XATan := ArcTan(E1) / PiDiv;
end;

Function XASin(E1: Real): Real;  
begin
  If E1 = 1 then XASin := 90 else
  If E1 = -1 then XASin := -90 else
  if not (1-Sqr(E1) > 0) then XASin := 0 else
  XASin := (ArcTan (E1 / sqrt (1-sqr (E1)))) / PiDiv;
end;

Function XACos(E1: Real): Real;  
begin
if not (1-Sqr(E1) > 0) then XACos := 0 else
try
  if E1 = 0 then XACos := 0 else
  if E1 > 0 then XACos := (ArcTan (sqrt (1-sqr (E1)) / E1)) / PiDiv
    else XAcos := ((ArcTan (sqrt (1-sqr (E1)) /E1)) / PiDiv) + 180;
except
  XACos := 0;
end;
end;

Function XALog(E1: Real): Real;  
begin
 { If E1 < -10 then
  XALog := (Exp(E1 * LogN10)) * -1 else }
  XALog := Exp(E1 * LogN10);
end;

Function MParts(E1: Real): Real;  
var
  Com1: Real;
begin
  Com1 := MPartConst1 * Xlog(XTan(45 + 0.5 * Abs(E1)))
            - MPartConst2 * XSin(Abs(E1));
  if E1 < 0 then MParts := Com1 * -1 else MParts := Com1;
end;

Function MLati(E1: Real): Real;
var
  Lat, Com1: Real;
begin
  Lat := 2 * (XAtan(XAlog(E1 / MPartConst1)) - 45);
  Com1 := MPartConst2 * XSin(Lat);
  Lat := 2 * (XAtan(XAlog((E1 + Com1) / MPartConst1)) - 45);
  Com1 := MPartConst2 * XSin(Lat);
  Lat := 2 * (XAtan(XAlog((E1 + Com1) / MPartConst1)) - 45);
  Com1 := MPartConst2 * XSin(Lat);
  MLati := 2 * (XAtan(XAlog((E1 + Com1) / MPartConst1)) - 45);
end;

end.
 